@@ -48,22 +48,31 @@ parse_request_headers (SoupMessage *msg, char *headers, guint headers_len,
 	/* Handle request body encoding */
 	*encoding = soup_message_headers_get_encoding (msg->request_headers);
 	if (*encoding == SOUP_ENCODING_UNRECOGNIZED) {
-		if (soup_message_headers_get (msg->request_headers, "Transfer-Encoding"))
+		if (soup_message_headers_get_list (msg->request_headers, "Transfer-Encoding"))
 			return SOUP_STATUS_NOT_IMPLEMENTED;
 		else
 			return SOUP_STATUS_BAD_REQUEST;
 	}
 
 	/* Generate correct context for request */
-	req_host = soup_message_headers_get (msg->request_headers, "Host");
+	req_host = soup_message_headers_get_one (msg->request_headers, "Host");
+	if (req_host && strchr (req_host, '/')) {
+		g_free (req_path);
+		return SOUP_STATUS_BAD_REQUEST;
+	}
 
-	if (*req_path != '/') {
-		/* Check for absolute URI */
+	if (!strcmp (req_path, "*") && req_host) {
+		/* Eg, "OPTIONS * HTTP/1.1" */
+		url = g_strdup_printf ("%s://%s",
+				       soup_socket_is_ssl (sock) ? "https" : "http",
+				       req_host);
+		uri = soup_uri_new (url);
+		if (uri)
+			soup_uri_set_path (uri, "*");
+		g_free (url);
+	} else if (*req_path != '/') {
+		/* Must be an absolute URI */
 		uri = soup_uri_new (req_path);
-		if (!uri) {
-			g_free (req_path);
-			return SOUP_STATUS_BAD_REQUEST;
-		}
 	} else if (req_host) {
 		url = g_strdup_printf ("%s://%s%s",
 				       soup_socket_is_ssl (sock) ? "https" : "http",
@@ -154,8 +163,8 @@ handle_partial_get (SoupMessage *msg)
 		 */
 
 		multipart = soup_multipart_new ("multipart/byteranges");
-		content_type = soup_message_headers_get (msg->response_headers,
-							 "Content-Type");
+		content_type = soup_message_headers_get_one (msg->response_headers,
+							     "Content-Type");
 		for (i = 0; i < nranges; i++) {
 			part_headers = soup_message_headers_new (SOUP_MESSAGE_HEADERS_MULTIPART);
 			if (content_type) {
