@@ -166,6 +166,13 @@ finalize (GObject *object)
 	if (priv->proxy_auth)
 		g_object_unref (priv->proxy_auth);
 
+	g_slist_free (priv->disabled_features);
+
+	while (priv->decoders) {
+		g_object_unref (priv->decoders->data);
+		priv->decoders = g_slist_delete_link (priv->decoders, priv->decoders);
+	}
+
 	soup_message_body_free (msg->request_body);
 	soup_message_headers_free (msg->request_headers);
 	soup_message_body_free (msg->response_body);
@@ -1015,7 +1022,6 @@ soup_message_add_header_handler (SoupMessage *msg,
 				 GCallback    callback,
 				 gpointer     user_data)
 {
-	SoupMessagePrivate *priv;
 	GClosure *closure;
 	char *header_name;
 
@@ -1024,8 +1030,6 @@ soup_message_add_header_handler (SoupMessage *msg,
 	g_return_val_if_fail (header != NULL, 0);
 	g_return_val_if_fail (callback != NULL, 0);
 
-	priv = SOUP_MESSAGE_GET_PRIVATE (msg);
-
 	closure = g_cclosure_new (callback, user_data, NULL);
 
 	header_name = g_strdup (header);
@@ -1225,6 +1229,12 @@ soup_message_cleanup_response (SoupMessage *req)
 						   SOUP_ENCODING_CONTENT_LENGTH);
 	}
 
+	while (priv->decoders) {
+		g_object_unref (priv->decoders->data);
+		priv->decoders = g_slist_delete_link (priv->decoders, priv->decoders);
+	}
+	priv->msg_flags &= ~SOUP_MESSAGE_CONTENT_DECODED;
+
 	req->status_code = SOUP_STATUS_NONE;
 	if (req->reason_phrase) {
 		g_free (req->reason_phrase);
@@ -1235,6 +1245,7 @@ soup_message_cleanup_response (SoupMessage *req)
 	g_object_notify (G_OBJECT (req), SOUP_MESSAGE_STATUS_CODE);
 	g_object_notify (G_OBJECT (req), SOUP_MESSAGE_REASON_PHRASE);
 	g_object_notify (G_OBJECT (req), SOUP_MESSAGE_HTTP_VERSION);
+	g_object_notify (G_OBJECT (req), SOUP_MESSAGE_FLAGS);
 }
 
 /**
@@ -1367,14 +1378,13 @@ soup_message_is_keepalive (SoupMessage *msg)
 		return FALSE;
 
 	if (SOUP_MESSAGE_GET_PRIVATE (msg)->http_version == SOUP_HTTP_1_0) {
-		/* Only persistent if the client requested keepalive
-		 * and the server agreed.
+		/* In theory, HTTP/1.0 connections are only persistent
+		 * if the client requests it, and the server agrees.
+		 * But some servers do keep-alive even if the client
+		 * doesn't request it. So ignore c_conn.
 		 */
 
-		if (!c_conn || !s_conn)
-			return FALSE;
-		if (!soup_header_contains (c_conn, "Keep-Alive") ||
-		    !soup_header_contains (s_conn, "Keep-Alive"))
+		if (!s_conn || !soup_header_contains (s_conn, "Keep-Alive"))
 			return FALSE;
 	} else {
 		/* Normally persistent unless either side requested otherwise */
@@ -1601,3 +1611,53 @@ soup_message_set_chunk_allocator (SoupMessage *msg,
 	priv->chunk_allocator_data    = user_data;
 	priv->chunk_allocator_dnotify = destroy_notify;
 }
+
+/**
+ * soup_message_disable_feature:
+ * @msg: a #SoupMessage
+ * @feature_type: the #GType of a #SoupSessionFeature
+ *
+ * This disables the actions of #SoupSessionFeature<!-- -->s with the
+ * given @feature_type (or a subclass of that type) on @msg, so that
+ * @msg is processed as though the feature(s) hadn't been added to the
+ * session. Eg, passing #SOUP_TYPE_PROXY_RESOLVER for @feature_type
+ * will disable proxy handling and cause @msg to be sent directly to
+ * the indicated origin server, regardless of system proxy
+ * configuration.
+ *
+ * You must call this before queueing @msg on a session; calling it on
+ * a message that has already been queued is undefined. In particular,
+ * you cannot call this on a message that is being requeued after a
+ * redirect or authentication.
+ *
+ * Since: 2.28
+ **/
+void
+soup_message_disable_feature (SoupMessage *msg, GType feature_type)
+{
+	SoupMessagePrivate *priv;
+
+	g_return_if_fail (SOUP_IS_MESSAGE (msg));
+
+	priv = SOUP_MESSAGE_GET_PRIVATE (msg);
+
+	priv->disabled_features = g_slist_prepend (priv->disabled_features,
+						   GSIZE_TO_POINTER (feature_type));
+}
+
+gboolean
+soup_message_disables_feature (SoupMessage *msg, gpointer feature)
+{
+	SoupMessagePrivate *priv;
+	GSList *f;
+
+	g_return_val_if_fail (SOUP_IS_MESSAGE (msg), FALSE);
+
+	priv = SOUP_MESSAGE_GET_PRIVATE (msg);
+
+	for (f = priv->disabled_features; f; f = f->next) {
+		if (G_TYPE_CHECK_INSTANCE_TYPE (feature, (GType) GPOINTER_TO_SIZE (f->data)))
+			return TRUE;
+	}
+	return FALSE;
+}
